import os
import json

import requests
from fastmcp import FastMCP
from pydantic import Field

HA_NOTIFY_DATA_PROMPT = """
```json
{
  "image": "http://a.com/photo.jpg",
  "video": "http://a.com/video.mp4",
  "audio": "http://a.com/audio.mp3", # ios only
  "actions": [
    {
      "action": "YOUR_ACTION_KEY", # Required. The identifier passed back in events.
      "title": "Do Something",
      "icon": "sfsymbols:bell.slash" # ios only
    },
    {
      "action": "URL", # Must be set to URI if you plan to use a URI
      "title": "Open Url", # The action button title
      "url": "https://github.com" # URL to open when action is selected
    },
    {
      "action": "REPLY", # When set to REPLY, you will be prompted for text to send with the event.
      "title": "Reply me",
      "behavior": "textInput" # Optional. Set to `textInput` to prompt for text to return with the event. This also occurs when setting the action to `REPLY`.
    }
  ]
}
```
All fields in the extended data are optional.
"""

def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="Send to HomeAssistant Mobile APP",
        description="Send a notification to Home Assistant Mobile APP",
    )
    def ha_send_mobile(
        message: str = Field(description="Notification content"),
        title: str = Field("", description="Notification title"),
        subtitle: str = Field("", description="Notification subtitle"),
        data: str | dict = Field("{}", description=f"Extended data, json string.{HA_NOTIFY_DATA_PROMPT}"),
        url: str = Field("", description="Opening a URL when tapping on a notification"),
        device_key: str = Field("", description="Device key, Default to get from environment variables"),
    ):
        base = os.getenv("HASS_BASE_URL") or "http://homeassistant.local:8123"
        if not (token := os.getenv("HASS_ACCESS_TOKEN")):
            return "You need to set `HASS_ACCESS_TOKEN` in the environment variable"

        headers = {"Authorization": f"Bearer {token}"}
        if not device_key:
            device_key = os.getenv("HASS_MOBILE_KEY", "")
        if not device_key:
            res = requests.get(f"{base}/api/services", headers=headers)
            for service in res.json() or []:
                if service["domain"] != "notify":
                    continue
                for name in service["services"]:
                    if name.startswith("mobile_app_"):
                        device_key = name
                        break
        if device_key.startswith("notify."):
            device_key = device_key[7:]
        elif not device_key.startswith("mobile_app_"):
            device_key = f"mobile_app_{device_key}"

        if isinstance(data, str):
            try:
                data = json.loads(data)
            except ValueError:
                data = {}
        elif not isinstance(data, dict):
            data = {}
        if url:
            data.setdefault("url", url)  # ios
            data.setdefault("clickAction", url)  # android
        if subtitle:
            data.setdefault("subtitle", subtitle)  # ios
            data.setdefault("subject", subtitle)  # android

        res = requests.post(
            f"{base}/api/services/notify/{device_key}",
            json={
                "message": message,
                "title": title,
                "data": data,
            },
            headers=headers,
        )
        return res.json()



from fastmcp import FastMCP
from pydantic import Field
from urllib.parse import urlencode


def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="文本转音频",
        description="将一段文本转成mp3音频链接",
    )
    def text_to_sound(
        text: str = Field(description="文本内容"),
        lang: str = Field("en", description="目标语言，支持: en/zh/cte(粤语)/ara/de/fra/kor/pt/ru/spa/th, 建议根据文本内容选择"),
        speed: int = Field(7, description="语速，默认7"),
    ):
        if not text:
            return ""
        return 'https://fanyi.baidu.com/gettts?' + urlencode({
            'lan': lang,
            'spd': speed,
            'text': text,
            'source': 'web',
        })



import os
import io
import logging
import requests
import hashlib
import base64
from fastmcp import FastMCP
from pydantic import Field
from cachetools import cached, TTLCache

_LOGGER = logging.getLogger(__name__)

WEWORK_BOT_KEY = os.getenv("WEWORK_BOT_KEY", "")
WEWORK_APP_AGENTID = int(os.getenv("WEWORK_APP_AGENTID", 1000002))
WEWORK_APP_CORPID = os.getenv("WEWORK_APP_CORPID", "")
WEWORK_APP_SECRET = os.getenv("WEWORK_APP_SECRET", "")
WEWORK_APP_TOUSER = os.getenv("WEWORK_APP_TOUSER", "@all")
WEWORK_BASE_URL = os.getenv("WEWORK_BASE_URL") or "https://qyapi.weixin.qq.com"

FIELD_BOT_KEY = Field("", description="企业微信群机器人key，uuid格式，默认从环境变量获取")
FIELD_TO_USER = Field("", description="接收消息的成员ID，多个用`|`分隔，为`@all`时向该企业应用全部成员发送，默认从环境变量获取")


def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="企业微信群机器人-发送文本消息",
        description="通过企业微信群机器人发送文本或Markdown消息",
    )
    def wework_send_text(
        text: str = Field(description="消息内容，长度限制: (text: 2048个字节, markdown_v2: 4096个字节)"),
        msgtype: str = Field("text", description="内容类型，仅支持: text/markdown_v2"),
        bot_key: str = FIELD_BOT_KEY,
    ):
        if msgtype == "markdown":
            msgtype = "markdown_v2"
        res = requests.post(
            f"{WEWORK_BASE_URL}/cgi-bin/webhook/send?key={bot_key or WEWORK_BOT_KEY}",
            json={"msgtype": msgtype, msgtype: {"content": text}},
        )
        return res.json()


    @mcp.tool(
        title="企业微信群机器人-发送图片消息",
        description="通过企业微信群机器人发送图片消息",
    )
    def wework_send_image(
        url: str = Field(description="图片url"),
        bot_key: str = FIELD_BOT_KEY,
    ):
        res = requests.get(url, timeout=120)
        res.raise_for_status()
        b64str = base64.b64encode(res.content).decode()
        md5str = hashlib.md5(res.content).hexdigest()
        res = requests.post(
            f"{WEWORK_BASE_URL}/cgi-bin/webhook/send?key={bot_key or WEWORK_BOT_KEY}",
            json={"msgtype": "image", "image": {"base64": b64str, "md5": md5str}},
            timeout=120,
        )
        return res.json()


    @mcp.tool(
        title="企业微信群机器人-发送图文消息",
        description="通过企业微信群机器人发送图文链接消息",
    )
    def wework_send_news(
        title: str = Field(description="标题，不超过128个字节"),
        url: str = Field(description="跳转链接，必填"),
        picurl: str = Field("", description="图片URL"),
        description: str = Field("", description="描述，不超过512个字节"),
        bot_key: str = FIELD_BOT_KEY,
    ):
        res = requests.post(
            f"{WEWORK_BASE_URL}/cgi-bin/webhook/send?key={bot_key or WEWORK_BOT_KEY}",
            json={
                "msgtype": "news",
                "news": {
                    "articles": [
                        {
                            "title": title,
                            "description": description,
                            "url": url,
                            "picurl": picurl,
                        },
                    ],
                },
            },
        )
        return res.json()


    if WEWORK_APP_CORPID and WEWORK_APP_SECRET:
        @cached(TTLCache(maxsize=1, ttl=3600))
        def get_access_token():
            res = requests.get(
                f"{WEWORK_BASE_URL}/cgi-bin/gettoken",
                params={"corpid": WEWORK_APP_CORPID, "corpsecret": WEWORK_APP_SECRET},
                timeout=60,
            )
            return res.json().get("access_token")


        @mcp.tool(
            title="企业微信应用号-发送文本消息",
            description="通过企业微信应用号发送文本或Markdown消息",
        )
        def wework_app_send_text(
            text: str = Field(description="消息内容，最长不超过2048个字节"),
            msgtype: str = Field("text", description="内容类型，仅支持: text/markdown"),
            touser: str = FIELD_TO_USER,
        ):
            res = requests.post(
                f"{WEWORK_BASE_URL}/cgi-bin/message/send?access_token={get_access_token()}",
                json={
                    "touser": touser or WEWORK_APP_TOUSER,
                    "agentid": WEWORK_APP_AGENTID,
                    "msgtype": msgtype,
                    msgtype: {"content": text},
                    "enable_duplicate_check": 1,
                    "duplicate_check_interval": 60,
                },
            )
            return res.json() or {}

        @mcp.tool(
            title="企业微信应用号-发送图片消息",
            description="通过企业微信应用号发送发送图片消息",
        )
        def wework_app_send_image(
            url: str = Field(description="图片URL"),
            touser: str = FIELD_TO_USER,
        ):
            return wework_send_media(touser, url, "image")

        @mcp.tool(
            title="企业微信应用号-发送视频消息",
            description="通过企业微信应用号发送发送视频消息",
        )
        def wework_app_send_video(
            url: str = Field(description="视频URL"),
            touser: str = FIELD_TO_USER,
        ):
            return wework_send_media(touser, url, "video")

        @mcp.tool(
            title="企业微信应用号-发送语音消息",
            description="通过企业微信应用号发送发送语音消息",
        )
        def wework_app_send_voice(
            url: str = Field(description="语音URL"),
            touser: str = FIELD_TO_USER,
        ):
            return wework_send_media(touser, url, "voice")

        @mcp.tool(
            title="企业微信应用号-发送文件消息",
            description="通过企业微信应用号发送发送文件消息",
        )
        def wework_app_send_file(
            url: str = Field(description="文件URL"),
            touser: str = FIELD_TO_USER,
        ):
            return wework_send_media(touser, url, "file")

        def wework_send_media(touser, url: str, msgtype=None):
            if msgtype:
                pass
            elif '.jpg' in url.lower() or '.jpeg' in url.lower() or '.png' in url.lower():
                msgtype = 'image'
            elif '.mp4' in url.lower():
                msgtype = 'video'
            elif '.arm' in url.lower():
                msgtype = 'voice'
            else:
                msgtype = 'file'
            res = requests.get(url, timeout=120)
            res.raise_for_status()
            file = io.BytesIO(res.content)
            mine = res.headers.get("content-type") or "application/octet-stream"
            res = requests.post(
                f"{WEWORK_BASE_URL}/cgi-bin/media/upload",
                params={"type": msgtype, "access_token": get_access_token()},
                files={"media": ("filename", file, mine)},
                timeout=120,
            )
            media = res.json() or {}
            if not (media_id := media.get("media_id")):
                return media
            res = requests.post(
                f"{WEWORK_BASE_URL}/cgi-bin/message/send?access_token={get_access_token()}",
                json={
                    "touser": touser or WEWORK_APP_TOUSER,
                    "agentid": WEWORK_APP_AGENTID,
                    "msgtype": msgtype,
                    msgtype: {"media_id": media_id},
                },
            )
            return res.json()


        @mcp.tool(
            title="企业微信应用号-发送图文卡片消息",
            description="通过企业微信应用号发送图文卡片消息",
        )
        def wework_app_send_news(
            title: str = Field(description="标题，不超过128个字符"),
            url: str = Field(description="跳转链接，最长2048字节，必须包含协议头(http/https)"),
            picurl: str = Field("", description="图片URL"),
            description: str = Field("", description="描述，不超过512个字符"),
            touser: str = FIELD_TO_USER,
        ):
            res = requests.post(
                f"{WEWORK_BASE_URL}/cgi-bin/message/send?access_token={get_access_token()}",
                json={
                    "touser": touser or WEWORK_APP_TOUSER,
                    "agentid": WEWORK_APP_AGENTID,
                    "msgtype": "news",
                    "news": {
                        "articles": [
                            {
                                "title": title,
                                "description": description,
                                "url": url,
                                "picurl": picurl,
                            },
                        ],
                    },
                },
            )
            return res.json() or {}



import os
import logging
import requests
from fastmcp import FastMCP
from pydantic import Field

_LOGGER = logging.getLogger(__name__)

NTFY_ACTIONS_RULE = """
The following actions are supported:
- view: Opens a website or app when the action button is tapped
- broadcast: Sends an Android broadcast intent when the action button is tapped (only supported on Android)
- http: Sends HTTP POST/GET/PUT request when the action button is tapped
```json
[
  {
    "action": "view",
    "label": "Open portal",
    "url": "https://home.nest.com/",
    "clear": true
  },
  {
    "action": "http",
    "label": "Turn down",
    "url": "https://api.nest.com/",
    "body": "{\"temperature\": 65}"
  },
  {
    "action": "broadcast",
    "label": "Take picture",
    "extras": {
        "cmd": "pic",
        "camera": "front"
    }
  }
]
```
"""


def add_tools(mcp: FastMCP):

    @mcp.tool(
        title="钉钉群机器人-发送文本消息",
        description="钉钉群机器人发送文本或Markdown消息",
    )
    def ding_send_text(
        text: str = Field(description="消息内容"),
        title: str = Field("", description="消息标题"),
        msgtype: str = Field("markdown", description="内容类型，仅支持: text/markdown"),
        bot_key: str = Field("", description="钉钉群机器人access_token，默认从环境变量获取"),
    ):
        """
        https://open.dingtalk.com/document/development/custom-robots-send-group-messages
        """
        if msgtype == "markdown":
            body = {"title": title, "text": text}
        else:
            body = {"content": f'{title}\n{text}'.strip()}
        if not bot_key:
            bot_key = os.getenv("DINGTALK_BOT_KEY", "")
        base = os.getenv("DINGTALK_BASE_URL") or "https://oapi.dingtalk.com"
        res = requests.post(
            f"{base}/robot/send?access_token={bot_key}",
            json={"msgtype": msgtype, msgtype: body},
        )
        return res.json()


    @mcp.tool(
        title="飞书/Lark机器人-发送文本消息",
        description="飞书/Lark群机器人发送文本或Markdown消息",
    )
    def lark_send_text(
        text: str = Field(description="消息内容"),
        msgtype: str = Field("markdown", description="内容类型，仅支持: text/markdown"),
        bot_key: str = Field("", description="飞书/Lark机器人key，uuid格式，默认从环境变量获取"),
        is_lark: int = Field(0, description="根据用户描述识别 0:飞书 1:Lark"),
    ):
        """
        https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN
        https://open.larksuite.com/document/client-docs/bot-v3/add-custom-bot
        """
        if msgtype == "markdown":
            body = {
                "msg_type": "interactive",
                "card": {"elements": [{"tag": msgtype, "content": text}]},
            }
        else:
            body = {"msg_type": msgtype, "content": {"text": text}}
        if not bot_key:
            bot_key = os.getenv("LARK_BOT_KEY" if is_lark else "FEISHU_BOT_KEY", "")
        if is_lark:
            base = os.getenv("LARK_BASE_URL") or "https://open.larksuite.com"
        else:
            base = os.getenv("FEISHU_BASE_URL") or "https://open.feishu.cn"
        res = requests.post(
            f"{base}/open-apis/bot/v2/hook/{bot_key}",
            json=body,
        )
        return res.json()


    @mcp.tool(
        title="Bark推送通知",
        description="通过Bark推送通知",
    )
    def bark_send_notify(
        body: str = Field(description="推送内容"),
        title: str = Field("", description="推送标题"),
        subtitle: str = Field("", description="推送副标题"),
        device_key: str = Field("", description="设备key，默认从环境变量获取"),
        url: str = Field("", description="点击推送时，跳转的URL ，支持URL Scheme 和 Universal Link"),
        icon: str = Field("", description="自定义图标URL"),
        level: str = Field(
            "active",
            description="推送中断级别。"
                        "critical: 重要警告, 在静音模式下也会响铃"
                        "active：默认值，系统会立即亮屏显示通知"
                        "timeSensitive：时效性通知，可在专注状态下显示通知。"
                        "passive：仅将通知添加到通知列表，不会亮屏提醒。",
        ),
        volume: int = Field(5, description="重要警告的通知音量(0-10)，默认为5"),
    ):
        """
        https://bark.day.app/#/tutorial
        """
        if not device_key:
            device_key = os.getenv("BARK_DEVICE_KEY", "")
        base = os.getenv("BARK_BASE_URL") or "https://api.day.app"
        res = requests.post(
            f"{base}/{device_key}",
            json={
                "body": body,
                "title": title,
                "subtitle": subtitle,
                "url": url,
                "icon": icon,
                "level": level,
                "volume": volume,
            },
        )
        return res.json()


    @mcp.tool(
        title="Ntfy Push Notification",
        description="Push a notification via Ntfy",
    )
    def ntfy_send_notify(
        message: str = Field(description="Notification message body; set to `triggered` if empty or not passed"),
        title: str = Field("", description="Notification title"),
        topic: str = Field("", description="Target topic name or URL"),
        click: str = Field("", description="URL opened when notification is clicked"),
        attach: str = Field("", description="URL of an attachment"),
        icon: str = Field("", description="URL of notification icon"),
        markdown: bool = Field(False, description="Set to `true` if the message is Markdown-formatted"),
        filename: str = Field("", description="File name of the attachment"),
        priority: int = Field(3, description="Message priority with 1=min, 3=default and 5=max"),
        delay: str = Field("", description="Timestamp or duration for delayed delivery. Example: 30min, 9am"),
        actions: list | None = Field(None, description=f"List of action buttons.{NTFY_ACTIONS_RULE}"),
    ):
        """
        https://docs.ntfy.sh/publish/#publish-as-json
        """
        base = os.getenv("NTFY_BASE_URL") or "https://ntfy.sh"
        if topic and topic.startswith("http"):
            base, topic = topic.rsplit("/", 1)
        if not topic:
            topic = os.getenv("NTFY_DEFAULT_TOPIC", "")
        data = {
            "topic": topic,
            "title": title,
            "message": message,
            "click": click,
            "icon": icon,
            "markdown": markdown or False,
            "priority": priority,
            "delay": delay,
        }
        if attach:
            data["attach"] = attach
            data["filename"] = filename
        if actions:
            data["actions"] = actions
        res = requests.post(f"{base}", json=data)
        return res.json()


    @mcp.tool(
        title="PushPlus推送消息",
        description="通过PushPlus(推送加)推送消息",
    )
    def pushplus_send_msg(
        content: str = Field(description="消息内容"),
        title: str = Field("", description="消息标题"),
        token: str = Field("", description="用户token，默认从环境变量获取"),
        template: str = Field("", description="消息内容格式: `html`(默认)/`txt`/`markdown`"),
        channel: str = Field("", description="发送渠道: `wechat`(默认)/`webhook`/`mail`"),
    ):
        """
        https://www.pushplus.plus/doc/guide/api.html
        """
        if not token:
            token = os.getenv("PUSH_PLUS_TOKEN", "")
        base = os.getenv("PUSH_PLUS_BASE_URL") or "http://www.pushplus.plus"
        res = requests.post(
            f"{base}/{token}",
            json={
                "content": content,
                "title": title,
                "template": template,
                "channel": channel,
            },
        )
        return res.json()
